import unittest

from .cn2an import Cn2An


class Cn2anTest(unittest.TestCase):
    def setUp(self):
        self.error_input_data = [
            "零点点",
            "零点零大"
        ]
        self.integer_data = {
            "一": 1,
            "壹": 1,
            "十一": 11,
            "一十一": 11,
            "一百万零五十四": 1000054,
            "壹佰万零伍十肆": 1000054,
            "三千壹佰万零伍十肆": 31000054
        }
        self.integer_decimal_data = {
            # python 默认只有16位的有效精度
            "零点零零零五零零零五零零零五零零零五零": 0.000500050005,
            "零点零零零五零零零五": 0.00050005,
            "零点零零零零五零": 0.00005,
            "零点零零零零五": 0.00005,
            "零点零零": 0,
            "零点零": 0,
            "一百万零五十四点四三二一": 1000054.4321,
            "一百万零五十四点": 1000054,
            "点四三二一": 0.4321
        }

        self.ca = Cn2An()
    
    def test_cn2an(self):
        with self.assertRaises(ValueError):
            for error_data in self.error_input_data:
                self.ca.cn2an(error_data)
        
        for integer_item in self.integer_data.keys():
            self.assertEqual(self.ca.cn2an(integer_item), self.integer_data[integer_item])
        
        for integer_decimal_item in self.integer_decimal_data.keys():
            self.assertEqual(self.ca.cn2an(integer_decimal_item), self.integer_decimal_data[integer_decimal_item])


if __name__ == '__main__':
    unittest.main()


import sys

from . import utils


class An2Cn():
    def __init__(self):
        self.conf = utils.get_default_conf()
    
    def an2cn(self, input_data=u"defalut_key", is_cap=False):
        # 从命令行接受参数
        input_data, is_cap = self.input_from_terminal(input_data, is_cap)

        # 将数字转化为字符串
        if not isinstance(input_data, str):
            input_data = self.convert_number_to_string(input_data)

        # 检查数据是否有效
        self.check_input_data_is_valid(input_data)

        # 切割整数部分和小数部分
        split_result = input_data.split(".")
        len_split_result = len(split_result)
        if len_split_result == 1:
            # 不包含小数的输入
            integer_data = split_result[0]
            output = self.integer_convert(integer_data, is_cap)
        elif len_split_result == 2:
            # 包含小数的输入
            integer_data = split_result[0]
            decimal_data = split_result[1]
            output = self.integer_convert(integer_data, is_cap) + self.decimal_convert(decimal_data, is_cap)
        else:
            raise ValueError(u"输入格式错误：{}！".format(input_data))

        return output

    @staticmethod
    def input_from_terminal(input_data, is_cap):
        len_argv = len(sys.argv)
        if len_argv == 1:
            if input_data == u"defalut_key":
                raise Exception(u"请在an2cn后输入需要转化的阿拉伯数字！")
            else:
                return input_data, is_cap
        elif len_argv == 2:
            input_an = sys.argv[1]
        elif len_argv == 3:
            input_an = sys.argv[1]
            if sys.argv[2] == u"cap":
                is_cap = True
            else:
                raise Exception(u"参数2错误，如果你想使用大写功能，参数2应为cap！")
        else:
            raise Exception(u"an2cn后的参数过多！")

        return input_an, is_cap

    @staticmethod
    def check_input_data_is_valid(check_data):
        # 检查输入数据是否在规定的字典中
        all_check_keys = [u"0", u"1", u"2", u"3", u"4", u"5", u"6", u"7", u"8", u"9", u"."]
        for data in check_data:
            if data not in all_check_keys:
                raise ValueError(u"输入的数据不在转化范围内：{}！".format(data))
    
    @staticmethod
    def convert_number_to_string(number_data):
        # python 会自动把 0.00005 转化成 5e-05，因此 str(0.00005) != "0.00005"
        string_data = str(number_data)
        if u"e" in string_data:
            string_data_list = string_data.split(u"e")
            string_key = string_data_list[0]
            string_value = string_data_list[1]
            if string_value[0] == u"-":
                string_data = u"0." + u"0"*(int(string_value[1:])-1) + string_key
            else:
                string_data = string_key + u"0"*int(string_value)

        return string_data

    def integer_convert(self, integer_data, is_cap):
        if is_cap:
            numeral_list = self.conf["number_capital"]
            unit_list = self.conf["unit_capital"]
        else:
            numeral_list = self.conf["number"]
            unit_list = self.conf["unit"]
            
        output_an = u""
        len_integer_data = len(integer_data)

        for i, d in enumerate(integer_data):
            if int(d):
                output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]
            else:
                if not (len_integer_data - i - 1) % 4:
                    output_an += numeral_list[int(d)] + unit_list[len_integer_data - i - 1]
                
                if i > 0 and not output_an[-1] == u"零":
                    output_an += numeral_list[int(d)]

        output_an = output_an.replace(u"零零", u"零").replace(u"零万", u"万").replace(u"零亿", u"亿").replace(u"亿万", u"亿").strip(u"零")

        # 解决「一十几」和「壹拾几」问题
        if output_an[:2] in [u"一十", u"壹拾"]:
            output_an = output_an[1:]
            
        # 0 - 1 之间的小数
        if not output_an:
            output_an = u"零"
            
        return output_an


    def decimal_convert(self, decimal_data, is_cap):
        len_decimal_data = len(decimal_data) 
        
        if len_decimal_data > 15:
            print("warning: 小数部分长度为{}，超过15位有效精度长度，将自动截取前15位！".format(len_decimal_data))
            decimal_data = decimal_data[:15]

        if is_cap:
            numeral_list = self.conf[u"number_capital"]
        else:
            numeral_list = self.conf[u"number"]

        output_an = u"点"
        for data in decimal_data:
            output_an += numeral_list[int(data)]

        return output_an



import sys

from . import utils


class Cn2An():
    def __init__(self):
        self.conf = utils.get_default_conf()

    def cn2an(self, input_data=u"default_key"):
        # 从命令行接受参数
        input_data = self.input_from_terminal(input_data)

        if input_data:
            # 检查数据是否有效
            self.check_input_data_is_valid(input_data)
            
            # 切割整数部分和小数部分
            split_result = input_data.split(u"点")
            len_split_result = len(split_result)
            if len_split_result == 1:
                # 不包含小数的输入
                integer_data = split_result[0]
                output = self.integer_convert(integer_data)
            elif len_split_result == 2:
                # 包含小数的输入
                integer_data = split_result[0]
                decimal_data = split_result[1]
                output = self.integer_convert(integer_data) + self.decimal_convert(decimal_data)
            else:
                raise ValueError(u"输入格式错误：{}！".format(input_data))
        else:
            raise ValueError(u"输入数据为空！")
        
        if len(sys.argv) == 1:
            return output
        elif len(sys.argv) == 2:
            return str(output)
        else:
            raise ValueError("参数过多！")

    @staticmethod
    def input_from_terminal(input_data):
        len_argv = len(sys.argv)
        if len_argv == 1:
            if input_data == u"default_key":
                raise Exception(u"请在cn2an后输入需要转化的中文数字！")
            else:
                return input_data
        elif len_argv == 2:
            input_cn = sys.argv[1]
        else:
            raise Exception(u"cn2an后的参数过多！")

        return input_cn

    def check_input_data_is_valid(self, check_data):
        # 检查输入数据是否在规定的字典中
        all_check_keys = list(self.conf[u"number_unit"].keys())
        all_check_keys.append(u"点")
        for data in check_data:
            if data not in all_check_keys:
                raise ValueError(u"输入的数据不在转化范围内：{}！".format(data))
        
    def integer_convert(self, integer_data):
        output_integer = 0
        unit_value = 1
        ten_thousand_unit_key = 1
        for index in range(len(integer_data)-1, -1, -1):
            unit_key = self.conf[u"number_unit"].get(integer_data[index])

            if unit_key < 10:
                output_integer += unit_value * unit_key
            else:
                if unit_key % 10000 == 0:
                    if unit_key > ten_thousand_unit_key:
                        ten_thousand_unit_key = unit_key
                    else:
                        ten_thousand_unit_key = ten_thousand_unit_key * unit_key

                if unit_key > unit_value:
                    unit_value = unit_key
                else:
                    unit_value = ten_thousand_unit_key * unit_key

                if index == 0:
                    output_integer += unit_value

        return output_integer

    def decimal_convert(self, decimal_data):
        len_decimal_data = len(decimal_data) 
        
        if len_decimal_data > 15:
            print("warning: 小数部分长度为{}，超过15位有效精度长度，将自动截取前15位！".format(len_decimal_data))
            decimal_data = decimal_data[:15]
            len_decimal_data = 15
        
        output_decimal = 0
        for index in range(len(decimal_data)-1, -1, -1):
            unit_key = self.conf[u"number_unit"].get(decimal_data[index])
            output_decimal += unit_key * 10 ** -(index + 1)

        # 处理精度溢出问题
        output_decimal = round(output_decimal, len_decimal_data)

        return output_decimal



import unittest

from .an2cn import An2Cn


class An2CnTest(unittest.TestCase):
    def setUp(self):
        self.error_input_data = [
            u"123.1.1",
            u"0.1零"
        ]
        self.integer_data = {
            0: [u"零", u"零"],
            1: [u"一", u"壹"],
            11: [u"十一", u"拾壹"],
            1000000: [u"一百万", u"壹佰万"],
            1000054: [u"一百万零五十四", u"壹佰万零伍拾肆"],
            31000054: [u"三千一百万零五十四", u"叁仟壹佰万零伍拾肆"]
        }
        self.integer_decimal_data = {
            0.000500050005005: [u"零点零零零五零零零五零零零五零零五", u"零点零零零伍零零零伍零零零伍零零伍"],
            0.00005: [u"零点零零零零五", u"零点零零零零伍"],
            1000054.4321: [u"一百万零五十四点四三二一", u"壹佰万零伍拾肆点肆叁贰壹"],
            0.4321: [u"零点四三二一", u"零点肆叁贰壹"]
        }

        self.ac = An2Cn()
    
    def test_an2cn(self):
        with self.assertRaises(ValueError):
            for error_data in self.error_input_data:
                self.ac.an2cn(error_data)
        
        for integer_item in self.integer_data.keys():
            self.assertEqual(self.ac.an2cn(integer_item), self.integer_data[integer_item][0])
            self.assertEqual(self.ac.an2cn(integer_item, True), self.integer_data[integer_item][1])
        
        for integer_decimal_item in self.integer_decimal_data.keys():
            self.assertEqual(self.ac.an2cn(integer_decimal_item), self.integer_decimal_data[integer_decimal_item][0])
            self.assertEqual(self.ac.an2cn(integer_decimal_item, True), self.integer_decimal_data[integer_decimal_item][1])
        

if __name__ == '__main__':
    unittest.main()